package de.mcs.tools.sps.mnemonic;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertFalse;
import static org.junit.jupiter.api.Assertions.assertTrue;

import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;

import de.mcs.tools.sps.exceptions.IllegalArgument;
import de.mcs.tools.sps.exceptions.SyntaxError;

class TestDEQ {

  private static final int BYTE_VALUE_0 = 0xc8;
  private final String MNEMONIC_0 = "DEQ0";
  private static final int BYTE_VALUE_1 = 0xc4;
  private final String MNEMONIC_1 = "DEQ1";

  @Test
  void testMnemonic0() throws SyntaxError {
    DEQ0 mnemonic = new DEQ0(MNEMONIC_0 + " 0x01");
    mnemonic.checkArgument();
    assertEquals(BYTE_VALUE_0, mnemonic.getByte());
    assertTrue(mnemonic.hasArgument());
    assertFalse(mnemonic.isUsingPage());

    mnemonic = new DEQ0(MNEMONIC_0 + " 0x04");
    mnemonic.checkArgument();
    assertEquals(BYTE_VALUE_0 + 0x03, mnemonic.getByte());

    Assertions.assertThrows(SyntaxError.class, () -> {
      DEQ0 mno1 = new DEQ0(MNEMONIC_0);
      mno1.checkArgument();
    });

    Assertions.assertThrows(SyntaxError.class, () -> {
      DEQ0 mno1 = new DEQ0(MNEMONIC_0 + " 0x05");
      mno1.checkArgument();
    });

    Assertions.assertThrows(IllegalArgument.class, () -> {
      DEQ0 mno1 = new DEQ0(MNEMONIC_0 + " 0x10");
      mno1.checkArgument();
    });
  }

  @Test
  void testMnemonic1() throws SyntaxError {
    DEQ1 mnemonic = new DEQ1(MNEMONIC_1 + " 0x01");
    mnemonic.checkArgument();
    assertEquals(BYTE_VALUE_1, mnemonic.getByte());
    assertTrue(mnemonic.hasArgument());
    assertFalse(mnemonic.isUsingPage());

    mnemonic = new DEQ1(MNEMONIC_1 + " 0x04");
    mnemonic.checkArgument();
    assertEquals(BYTE_VALUE_1 + 0x03, mnemonic.getByte());

    Assertions.assertThrows(SyntaxError.class, () -> {
      DEQ1 mno1 = new DEQ1(MNEMONIC_1);
      mno1.checkArgument();
    });

    Assertions.assertThrows(SyntaxError.class, () -> {
      DEQ1 mno1 = new DEQ1(MNEMONIC_1 + " 0x05");
      mno1.checkArgument();
    });

    Assertions.assertThrows(IllegalArgument.class, () -> {
      DEQ1 mno1 = new DEQ1(MNEMONIC_1 + " 0x10");
      mno1.checkArgument();
    });
  }

  @Test
  void testMnemonic0Factory() throws SyntaxError {
    Mnemonic mnemonic = MnemonicFactory.getMnemonic(MNEMONIC_0 + " 1", 0);
    assertEquals(DEQ0.class, mnemonic.getClass());

    Assertions.assertThrows(SyntaxError.class, () -> {
      MnemonicFactory.getMnemonic(MNEMONIC_0 + " 0x00", 0);
    });

    Assertions.assertThrows(SyntaxError.class, () -> {
      MnemonicFactory.getMnemonic(MNEMONIC_0 + " 5", 0);
    });

    mnemonic = MnemonicFactory.getMnemonic(MNEMONIC_0 + " 0x02", 0);
    assertEquals(BYTE_VALUE_0 + 1, mnemonic.getByte());

    mnemonic = MnemonicFactory.getMnemonic(MNEMONIC_0 + " 3", 0);
    assertEquals(BYTE_VALUE_0 + 2, mnemonic.getByte());

    mnemonic = MnemonicFactory.getMnemonic(MNEMONIC_0 + " 0b00000100", 0);
    assertEquals(BYTE_VALUE_0 + 3, mnemonic.getByte());

  }

  @Test
  void testMnemonic1Factory() throws SyntaxError {
    Mnemonic mnemonic = MnemonicFactory.getMnemonic(MNEMONIC_1 + " 1", 0);
    assertEquals(DEQ1.class, mnemonic.getClass());

    Assertions.assertThrows(SyntaxError.class, () -> {
      MnemonicFactory.getMnemonic(MNEMONIC_1 + " 0x00", 0);
    });

    Assertions.assertThrows(SyntaxError.class, () -> {
      MnemonicFactory.getMnemonic(MNEMONIC_1 + " 5", 0);
    });

    mnemonic = MnemonicFactory.getMnemonic(MNEMONIC_1 + " 0x02", 0);
    assertEquals(BYTE_VALUE_1 + 1, mnemonic.getByte());

    mnemonic = MnemonicFactory.getMnemonic(MNEMONIC_1 + " 3", 0);
    assertEquals(BYTE_VALUE_1 + 2, mnemonic.getByte());

    mnemonic = MnemonicFactory.getMnemonic(MNEMONIC_1 + " 0b00000100", 0);
    assertEquals(BYTE_VALUE_1 + 3, mnemonic.getByte());

  }
}
